/************************************************************************
 
 Module:      Aver95m.cpp
 
 Project:     JTV
 
 Description: Aver95 remote control driver. WORKING !!!!!!!!!!!!!!
 
 Copyright (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>

 All rights reserved.
 
 You may not use any piece of the following code for commercial purposes
 without written permission from the author.
 
 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include <jtvrem.h>
#include <jtvbt8x8.h>
#include <btregs.h>
#include <jtvremsk.h>



KeyDef keys[]=
{
	{ 0x68, "Aver95m_TV", "" },
	{ 0xa8, "Aver95m_FM", "" },

	{ 0x48, "Aver95m_Exit", "Power", },

	{ 0x30, "Aver95m_Display", "", },
	{ 0x28, "Aver95m_Freeze", "", },
	{ 0x88, "Aver95m_Capture", "", },
	{ 0x38, "Aver95m_Video", "", },

	{ 0x80, "Aver95m_1", "1", },
	{ 0x40, "Aver95m_2", "2", },
	{ 0xc0, "Aver95m_3", "3", },
	{ 0x18, "Aver95m_Audio", },

	
	{ 0x20, "Aver95m_4", "4", },
	{ 0xa0, "Aver95m_5", "5", },
	{ 0x60, "Aver95m_6", "6", },
	{ 0xF0, "Aver95m_FullScreen", },


	{ 0xe0, "Aver95m_7", "7", },
	{ 0x10, "Aver95m_8", "8", },
	{ 0x90, "Aver95m_9", "9", },
	{ 0xc8, "Aver95m_Loop", },

	{ 0x50, "Aver95m_0", "0", },
	{ 0x00, "Aver95m_Left", "Left" },
	{ 0xe8, "Aver95m_Right", "Right" },
	{ 0x08, "Aver95m_Mute", "Mute" },


	{ 0xb0, "Aver95m_Color", ""},
	{ 0x70, "Aver95m_Preview", "" },
	{ 0xd8, "Aver95m_ChannelUp", "ChannelUp", },
	{ 0x58, "Aver95m_VolumeUp", "VolumeUp", },

	
	{ 0xb8, "Aver95m_AutoScan", ""},
	{ 0x98, "Aver95m_Enter", "Enter"},
	{ 0xf8, "Aver95m_ChannelDown", "ChannelDown", },
	{ 0x78, "Aver95m_VolumeDown", "VolumeDown", },
};


typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");  
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);
}

class CAver95 : public CRemoteDriver
{

public:

	CAver95(KeyDef *k, int count) : CRemoteDriver(k, count)
	{
 	  log("aver95m: Initializing aver95m.dll remote module.\n");
      log("aver95m: Copyright by Miroslaw Grybos, <mgrybos@postinfo.com.pl>\n");
    };


	void PollingThread()
	{
		IBT8x8Device *bt;
		DWORD last_code=0xffffffff;;
		DWORD last_tick=GetTickCount();
			
		if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
		{
			DWORD value;


			bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
			bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0x0004);

			bt->ReadBT(BT848_GPIO_OUT_EN, &value);
			bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00007000));

			bt->ReadBT(BT848_GPIO_DATA, &value);
			bt->WriteBT(BT848_GPIO_DATA, value | 0x7FC2);


			while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
			{

				Sleep(20);

				DWORD gpio_data=0;
				DWORD value=0;

				bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
	
           		if (gpio_data & 0x00010000)
        		{
                    if ((last_code == (gpio_data &0x00ff0000)>>16) && (GetTickCount() > last_tick + 200))
					{
						last_code=(gpio_data &0x00ff0000)>>16;
						AddStroke(last_code & 0xfa);
						last_tick = GetTickCount();

					} else
					{
	       			    last_code = (gpio_data &0x00ff0000)>>16;
					}
				}
        		else
        		{

					value = (gpio_data &0x00ff0000)>>16;
					if ((value!=last_code) && ((value & 0x1)==0) && (last_code!=0xffffffff))
					{
						last_code=value;
						AddStroke(value & 0xfb);
					}
					last_tick = GetTickCount();
				};


			};


			log("aver95m: Break event received. Bye!\n");

			bt->Release();
		};
	};
};

extern "C"
{
	__declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
	{
		return new CAver95(keys, sizeof(keys) / sizeof(keys[0]));
	};
};
